@@ -1,5 +1,3 @@
-// $Id: Timer_Wheel_T.cpp 97817 2014-07-28 17:27:29Z johnnyw $
-
 #ifndef ACE_TIMER_WHEEL_T_CPP
 #define ACE_TIMER_WHEEL_T_CPP
 
@@ -166,7 +164,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::open_i
 
 /// Destructor just cleans up its memory
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY>
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::~ACE_Timer_Wheel_T (void)
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::~ACE_Timer_Wheel_T ()
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::~ACE_Timer_Wheel_T");
 
@@ -184,7 +182,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::~ACE_Timer_Wheel_T (voi
 }
 
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> int
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::close (void)
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::close ()
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::close");
 
@@ -264,7 +262,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::find_node (long timer_i
 * @return True if empty
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> bool
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::is_empty (void) const
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::is_empty () const
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::is_empty");
   return timer_count_ == 0;
@@ -275,7 +273,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::is_empty (void) const
 * @return First (earliest) node in the wheel_'s earliest_spoke_ list
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> const ACE_Time_Value &
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::earliest_time (void) const
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::earliest_time () const
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::earliest_time");
   ACE_Timer_Node_T<TYPE>* n = this->get_first_i ();
@@ -302,7 +300,6 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::calculate_spoke
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> long
 ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::generate_timer_id (u_int spoke)
 {
-
   int cnt_bits = sizeof (long) * 8 - this->spoke_bits_;
   long max_cnt = ((long)1 << cnt_bits) - 1;
   if (spoke == this->spoke_count_)
@@ -313,24 +310,17 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::generate_timer_id (u_in
   if (root == root->get_next ())
     root->set_act(0);
 
-  // We use this field to keep track of the next counter value that
-  // may be in use. Of course it may have expired, so we just use
-  // this field so that we know when we don't have to check for duplicates
-#if defined (ACE_WIN64)
-  // The cast below is legit... we know that long is shorter than a
-  // pointer, but are only using it as a 'long' storage area.
-#  pragma warning(push)
-#  pragma warning(disable : 4311)
-#endif /* ACE_WIN64 */
-  long next_cnt = ACE_Utils::truncate_cast<long> ((intptr_t)root->get_act ());
-#if defined (ACE_WIN64)
-#  pragma warning(pop)
-#endif /* ACE_WIN64 */
-
   // This field is used as a counter instead of a timer_id.
   long cnt = root->get_timer_id ();
 
-  if (cnt >= max_cnt && root == root->get_next ())
+  if (cnt < max_cnt)
+    {
+      root->set_timer_id (cnt + 1);
+      return (cnt << this->spoke_bits_) | spoke;
+    }
+
+  // Count has overflowed its range.
+  if (root == root->get_next ())
     {
       // Special case when we overflow on an empty spoke. We can just
       // wrap the count around without searching for duplicates. We only
@@ -339,50 +329,20 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::generate_timer_id (u_in
       root->set_timer_id (1);
       return spoke;
     }
-  else if (cnt >= max_cnt)
-    { // overflow
-      cnt = 0; // try again starting at zero
-    }
-  else if (next_cnt == 0 || cnt < next_cnt)
-    {
-      root->set_timer_id (cnt + 1);
-      return (cnt << this->spoke_bits_) | spoke;
-    }
 
+  // Overflowed count, and the spoke is not empty. Search for an unused
+  // id value.
   //ACELIB_ERROR((LM_ERROR, "Timer id overflow. We have to search now.\n"));
-
-  // We've run out of consecutive id numbers so now we have to search
-  // for a unique id.
-  // We'll try increasing numbers until we find one that is not in use,
-  // and we'll record the next highest number so that we can avoid this
-  // search as often as possible.
-  for (; cnt < max_cnt - 1; ++cnt)
+  for (cnt = 0; cnt < max_cnt - 1; ++cnt)
     {
+      // Look for an unused id. Yes, every new id on this spoke will result in a
+      // scan until all the spoke's timers get canceled/expired then the spoke will
+      // start over like new. So, when an empty spot is found, don't reset the
+      // root node's timer_id - it stays at max until the spoke clears out and
+      // starts over.
       long id = (cnt << this->spoke_bits_) | spoke;
-      ACE_Timer_Node_T<TYPE>* n = this->find_spoke_node (spoke, id);
-      if (n == 0)
-        {
-          root->set_timer_id (cnt + 1);
-          // Now we need to find the next highest cnt in use
-          next_cnt = 0;
-          for (; n != root; n = n->get_next ())
-            {
-              long tmp = n->get_timer_id () >> this->spoke_bits_;
-              if (tmp > cnt && (tmp < next_cnt || next_cnt == 0))
-                next_cnt = tmp;
-            }
-#if defined (ACE_WIN64)
-          // The cast below is legit... we know we're storing a long in
-          // a pointer, but are only using it as a 'long' storage area.
-#  pragma warning(push)
-#  pragma warning(disable : 4312)
-#endif /* ACE_WIN64 */
-          root->set_act (reinterpret_cast<void*> (next_cnt));
-#if defined (ACE_WIN64)
-#  pragma warning(pop)
-#endif /* ACE_WIN64 */
-          return id;
-        }
+      if (0 == this->find_spoke_node (spoke, id))
+        return id;
     }
 
   return -1; // We did our best, but the spoke is full.
@@ -423,6 +383,10 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::schedule_i (const TYPE&
           n->set (type, act, future_time, interval, 0, 0, id);
           this->schedule_i (n, spoke, future_time);
         }
+      else
+        {
+          this->free_node (n);
+        }
       return id;
     }
 
@@ -707,7 +671,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::recalc_earliest
 * of the wheel.
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> void
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::dump (void) const
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::dump () const
 {
 #if defined (ACE_HAS_DUMP)
   ACE_TRACE ("ACE_Timer_Wheel_T::dump");
@@ -743,7 +707,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::dump (void) const
 * @return The earliest timer node.
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> ACE_Timer_Node_T<TYPE> *
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::remove_first (void)
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::remove_first ()
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::remove_first");
   return remove_first_expired (ACE_Time_Value::max_time);
@@ -780,7 +744,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::remove_first_expired (c
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY>
 ACE_Timer_Node_T<TYPE>*
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first (void)
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first ()
 {
   ACE_TRACE ("ACE_Timer_Wheel_T::get_first");
   return this->get_first_i ();
@@ -788,7 +752,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first (void)
 
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY>
 ACE_Timer_Node_T<TYPE>*
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first_i (void) const
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first_i () const
 {
   ACE_Timer_Node_T<TYPE>* root = this->spokes_[this->earliest_spoke_];
   ACE_Timer_Node_T<TYPE>* first = root->get_next ();
@@ -803,7 +767,7 @@ ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::get_first_i (void) cons
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY>
 ACE_Timer_Queue_Iterator_T<TYPE> &
-ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::iter (void)
+ACE_Timer_Wheel_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::iter ()
 {
   this->iterator_->first ();
   return *this->iterator_;
@@ -894,7 +858,7 @@ ACE_Timer_Wheel_Iterator_T<TYPE,FUNCTOR,ACE_LOCK,TIME_POLICY>::ACE_Timer_Wheel_I
 * Destructor, at this level does nothing.
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY>
-ACE_Timer_Wheel_Iterator_T<TYPE,FUNCTOR,ACE_LOCK,TIME_POLICY>::~ACE_Timer_Wheel_Iterator_T (void)
+ACE_Timer_Wheel_Iterator_T<TYPE,FUNCTOR,ACE_LOCK,TIME_POLICY>::~ACE_Timer_Wheel_Iterator_T ()
 {
 }
 
@@ -908,7 +872,7 @@ ACE_Timer_Wheel_Iterator_T<TYPE,FUNCTOR,ACE_LOCK,TIME_POLICY>::~ACE_Timer_Wheel_
 * current_node_ would be 0.
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> void
-ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::first (void)
+ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::first ()
 {
   this->goto_next(0);
 }
@@ -918,7 +882,7 @@ ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::first (void)
 * Positions the iterator at the next node.
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> void
-ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::next (void)
+ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::next ()
 {
   if (this->isdone())
     return;
@@ -957,7 +921,7 @@ ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::goto_next (u_i
 * @return True when we there aren't any more items (when current_node_ == 0)
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> bool
-ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::isdone (void) const
+ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::isdone () const
 {
   return this->current_node_ == 0;
 }
@@ -967,7 +931,7 @@ ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::isdone (void)
 *         is empty
 */
 template <class TYPE, class FUNCTOR, class ACE_LOCK, typename TIME_POLICY> ACE_Timer_Node_T<TYPE> *
-ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::item (void)
+ACE_Timer_Wheel_Iterator_T<TYPE, FUNCTOR, ACE_LOCK, TIME_POLICY>::item ()
 {
   return this->current_node_;
 }
